@@ -9,21 +9,39 @@ import sys
 import traceback
 import warnings
 
+import jsonschema
 import six
 import pandas as pd
 import numpy as np
 
-try:
-    from pandas.api.types import infer_dtype
-    # keywords required; see https://github.com/altair-viz/altair/issues/1314
-    _infer_dtype_kwds = {'skipna': False}
-except ImportError:
-    # This is the appropriate import for pandas < 0.20.0
-    from pandas.lib import infer_dtype
-    _infer_dtype_kwds = {}  # no keywords allowed in pandas < 0.20
-
 from .schemapi import SchemaBase, Undefined
 
+try:
+    from pandas.api.types import infer_dtype as _infer_dtype
+except ImportError:
+    # Import for pandas < 0.20.0
+    from pandas.lib import infer_dtype as _infer_dtype
+
+
+def infer_dtype(value):
+    """Infer the dtype of the value.
+
+    This is a compatibility function for pandas infer_dtype,
+    with skipna=False regardless of the pandas version.
+    """
+    if not hasattr(infer_dtype, '_supports_skipna'):
+        try:
+            _ = _infer_dtype([1], skipna=False)
+        except TypeError:
+            # pandas < 0.21.0 don't support skipna keyword
+            infer_dtype._supports_skipna = False
+        else:
+            infer_dtype._supports_skipna = True
+    if infer_dtype._supports_skipna:
+        return _infer_dtype(value, skipna=False)
+    else:
+        return _infer_dtype(value)
+
 
 TYPECODE_MAP = {'ordinal': 'O',
                 'nominal': 'N',
@@ -73,7 +91,7 @@ def infer_vegalite_type(data):
     data: Numpy array or Pandas Series
     """
     # Otherwise, infer based on the dtype of the input
-    typ = infer_dtype(data, **_infer_dtype_kwds)
+    typ = infer_dtype(data)
 
     # TODO: Once this returns 'O', please update test_select_x and test_select_y in test_api.py
 
@@ -406,3 +424,86 @@ def display_traceback(in_ipython=True):
         ip.showtraceback(exc_info)
     else:
         traceback.print_exception(*exc_info)
+
+
+def infer_encoding_types(args, kwargs, channels):
+    """Infer typed keyword arguments for args and kwargs
+
+    Parameters
+    ----------
+    args : tuple
+        List of function args
+    kwargs : dict
+        Dict of function kwargs
+    channels : module
+        The module containing all altair encoding channel classes.
+
+    Returns
+    -------
+    kwargs : dict
+        All args and kwargs in a single dict, with keys and types
+        based on the channels mapping.
+    """
+    # Construct a dictionary of channel type to encoding name
+    # TODO: cache this somehow?
+    channel_objs = (getattr(channels, name) for name in dir(channels))
+    channel_objs = (c for c in channel_objs
+                    if isinstance(c, type) and issubclass(c, SchemaBase))
+    channel_to_name = {c: c._encoding_name for c in channel_objs}
+    name_to_channel = {}
+    for chan, name in channel_to_name.items():
+        chans = name_to_channel.setdefault(name, {})
+        key = 'value' if chan.__name__.endswith('Value') else 'field'
+        chans[key] = chan                       
+    
+    # First use the mapping to convert args to kwargs based on their types.
+    for arg in args:
+        if isinstance(arg, (list, tuple)) and len(arg) > 0:
+            type_ = type(arg[0])
+        else:
+            type_ = type(arg)
+
+        encoding = channel_to_name.get(type_, None)
+        if encoding is None:
+            raise NotImplementedError("positional of type {}"
+                                        "".format(type_))
+        if encoding in kwargs:
+            raise ValueError("encoding {} specified twice.".format(encoding))
+        kwargs[encoding] = arg
+
+    def _wrap_in_channel_class(obj, encoding):
+        try:
+            condition = obj['condition']
+        except (KeyError, TypeError):
+            pass
+        else:
+            if condition is not Undefined:
+                obj = obj.copy()
+                obj['condition'] = _wrap_in_channel_class(condition, encoding)
+
+        if isinstance(obj, SchemaBase):
+            return obj
+
+        if isinstance(obj, six.string_types):
+            obj = {'shorthand': obj}
+
+        if isinstance(obj, (list, tuple)):
+            return [_wrap_in_channel_class(subobj, encoding) for subobj in obj]
+
+        if encoding not in name_to_channel:
+            warnings.warn("Unrecognized encoding channel '{}'".format(encoding))
+            return obj
+        
+        classes = name_to_channel[encoding]
+        cls = classes['value'] if 'value' in obj else classes['field']
+
+        try:
+            # Don't force validation here; some objects won't be valid until
+            # they're created in the context of a chart.
+            return cls.from_dict(obj, validate=False)
+        except jsonschema.ValidationError:
+            # our attempts at finding the correct class have failed
+            return obj
+
+    return {encoding: _wrap_in_channel_class(obj, encoding)
+            for encoding, obj in kwargs.items()}
