@@ -4,6 +4,7 @@ import json
 
 import numpy as np
 import pandas as pd
+import six
 
 from .. import infer_vegalite_type, sanitize_dataframe
 
@@ -80,6 +81,20 @@ def test_sanitize_dataframe():
     assert df.equals(df2)
 
 
+def test_sanitize_dataframe_colnames():
+    df = pd.DataFrame(np.arange(12).reshape(4, 3))
+
+    # Test that RangeIndex is converted to strings
+    df = sanitize_dataframe(df)
+    assert [isinstance(col, six.string_types) for col in df.columns]
+
+    # Test that non-string columns result in an error
+    df.columns = [4, 'foo', 'bar']
+    with pytest.raises(ValueError) as err:
+        sanitize_dataframe(df)
+    assert str(err.value).startswith('Dataframe contains invalid column name: 4.')
+
+
 def test_sanitize_dataframe_timedelta():
     df = pd.DataFrame({'r': pd.timedelta_range(start='1 day', periods=4)})
     with pytest.raises(ValueError) as err:
