@@ -19,6 +19,10 @@ except ImportError:
     selenium = None
 
 
+def getargs(*args, **kwargs):
+    return args, kwargs
+
+
 OP_DICT = {
     'layer': operator.add,
     'hconcat': operator.or_,
@@ -26,6 +30,37 @@ OP_DICT = {
 }
 
 
+def _make_chart_type(chart_type):
+    data = pd.DataFrame({
+        'x': [28, 55, 43, 91, 81, 53, 19, 87],
+        'y': [43, 91, 81, 53, 19, 87, 52, 28],
+        'color': list('AAAABBBB'),
+    })
+    base = alt.Chart(data).mark_point().encode(
+        x='x',
+        y='y',
+        color='color',
+    )
+
+    if chart_type in ['layer', 'hconcat', 'vconcat', 'concat']:
+        func = getattr(alt, chart_type)
+        return func(base.mark_square(), base.mark_circle())
+    elif chart_type == 'facet':
+        return base.facet('color')
+    elif chart_type == 'facet_encoding':
+        return base.encode(facet='color')
+    elif chart_type == 'repeat':
+        return base.encode(
+            alt.X(alt.repeat(), type='quantitative')
+        ).repeat(
+            ['x', 'y']
+        )
+    elif chart_type == 'chart':
+        return base
+    else:
+        raise ValueError("chart_type='{}' is not recognized".format(chart_type))
+
+
 @pytest.fixture
 def basic_chart():
     data = pd.DataFrame({
@@ -111,28 +146,27 @@ def test_chart_infer_types():
     assert dct['encoding']['y']['type'] == 'ordinal'
 
 
-def test_multiple_encodings():
+@pytest.mark.parametrize('args, kwargs', [
+    getargs(detail=['value:Q', 'name:N'], tooltip=['value:Q', 'name:N']),
+    getargs(detail=['value', 'name'], tooltip=['value', 'name']),
+    getargs(alt.Detail(['value:Q', 'name:N']), alt.Tooltip(['value:Q', 'name:N'])),
+    getargs(alt.Detail(['value', 'name']), alt.Tooltip(['value', 'name'])),
+    getargs([alt.Detail('value:Q'), alt.Detail('name:N')],
+            [alt.Tooltip('value:Q'), alt.Tooltip('name:N')]),
+    getargs([alt.Detail('value'), alt.Detail('name')],
+            [alt.Tooltip('value'), alt.Tooltip('name')]),
+])
+def test_multiple_encodings(args, kwargs):
+    df = pd.DataFrame({
+        'value': [1, 2, 3],
+        'name': ['A', 'B', 'C'],
+    })
     encoding_dct = [{'field': 'value', 'type': 'quantitative'},
                     {'field': 'name', 'type': 'nominal'}]
-    chart1 = alt.Chart('data.csv').mark_point().encode(
-        detail=['value:Q', 'name:N'],
-        tooltip=['value:Q', 'name:N']
-    )
-
-    chart2 = alt.Chart('data.csv').mark_point().encode(
-        alt.Detail(['value:Q', 'name:N']),
-        alt.Tooltip(['value:Q', 'name:N'])
-    )
-
-    chart3 = alt.Chart('data.csv').mark_point().encode(
-        [alt.Detail('value:Q'), alt.Detail('name:N')],
-        [alt.Tooltip('value:Q'), alt.Tooltip('name:N')]
-    )
-
-    for chart in [chart1, chart2, chart3]:
-        dct = chart.to_dict()
-        assert dct['encoding']['detail'] == encoding_dct
-        assert dct['encoding']['tooltip'] == encoding_dct
+    chart = alt.Chart(df).mark_point().encode(*args, **kwargs)
+    dct = chart.to_dict()
+    assert dct['encoding']['detail'] == encoding_dct
+    assert dct['encoding']['tooltip'] == encoding_dct
 
 
 def test_chart_operations():
@@ -185,6 +219,16 @@ def test_selection_to_dict():
     ).to_dict()
 
 
+def test_selection_expression():
+    selection = alt.selection_single(fields=['value'])
+
+    assert isinstance(selection.value, alt.expr.Expression)
+    assert selection.value.to_dict() == '{0}.value'.format(selection.name)
+
+    assert isinstance(selection['value'], alt.expr.Expression)
+    assert selection['value'].to_dict() == "{0}['value']".format(selection.name)
+
+
 @pytest.mark.parametrize('format', ['html', 'json', 'png', 'svg'])
 @pytest.mark.skipif('not selenium')
 def test_save(format, basic_chart):
@@ -444,20 +488,6 @@ def test_resolve_methods():
     assert chart.resolve == alt.Resolve(scale=alt.ScaleResolveMap(x='shared', y='independent'))
 
 
-def test_layer_marks():
-    chart = alt.LayerChart().mark_point()
-    assert chart.mark == 'point'
-
-    chart = alt.LayerChart().mark_point(color='red')
-    assert chart.mark == alt.MarkDef('point', color='red')
-
-    chart = alt.LayerChart().mark_bar()
-    assert chart.mark == 'bar'
-
-    chart = alt.LayerChart().mark_bar(color='green')
-    assert chart.mark == alt.MarkDef('bar', color='green')
-
-
 def test_layer_encodings():
     chart = alt.LayerChart().encode(x='column:Q')
     assert chart.encoding.x == alt.X(shorthand='column:Q')
@@ -477,6 +507,34 @@ def test_add_selection():
     assert chart.selection == expected
 
 
+def test_repeat_add_selections():
+    base = alt.Chart('data.csv').mark_point()
+    selection = alt.selection_single()
+    chart1 = base.add_selection(selection).repeat(list('ABC'))
+    chart2 = base.repeat(list('ABC')).add_selection(selection)
+    assert chart1.to_dict() == chart2.to_dict()
+
+
+def test_facet_add_selections():
+    base = alt.Chart('data.csv').mark_point()
+    selection = alt.selection_single()
+    chart1 = base.add_selection(selection).facet('val:Q')
+    chart2 = base.facet('val:Q').add_selection(selection)
+    assert chart1.to_dict() == chart2.to_dict()
+
+
+@pytest.mark.parametrize('charttype', [alt.layer, alt.concat, alt.hconcat, alt.vconcat])
+def test_compound_add_selections(charttype):
+    base = alt.Chart('data.csv').mark_point()
+    selection = alt.selection_single()
+    chart1 = charttype(
+        base.add_selection(selection),
+        base.add_selection(selection)
+    )
+    chart2 = charttype(base, base).add_selection(selection)
+    assert chart1.to_dict() == chart2.to_dict()
+
+
 def test_selection_property():
     sel = alt.selection_interval()
     chart = alt.Chart('data.csv').mark_point().properties(
@@ -740,3 +798,45 @@ def test_layer_errors():
     with pytest.raises(ValueError) as err:
         alt.layer(simple_chart) + facet_chart2
     assert str(err.value) == "Faceted charts cannot be layered."
+
+
+@pytest.mark.parametrize(
+    'chart_type',
+    ['layer', 'hconcat', 'vconcat', 'concat', 'facet', 'facet_encoding', 'repeat']
+)
+def test_resolve(chart_type):
+    chart = _make_chart_type(chart_type)
+    chart = chart.resolve_scale(
+        x='independent',
+    ).resolve_legend(
+        color='independent'
+    ).resolve_axis(
+        y='independent'
+    )
+    dct = chart.to_dict()
+    assert dct['resolve'] == {
+        'scale': {'x': 'independent'},
+        'legend': {'color': 'independent'},
+        'axis': {'y': 'independent'},
+    }
+
+
+# TODO: test vconcat, hconcat, concat when schema allows them.
+# This is blocked by https://github.com/vega/vega-lite/issues/5261
+@pytest.mark.parametrize('chart_type', ['chart', 'layer', 'facet_encoding'])
+@pytest.mark.parametrize('facet_arg', [None, 'facet', 'row', 'column'])
+def test_facet(chart_type, facet_arg):
+    chart = _make_chart_type(chart_type)
+    if facet_arg is None:
+        chart = chart.facet('color:N', columns=2)
+    else:
+        chart = chart.facet(**{facet_arg: 'color:N', 'columns': 2})
+    dct = chart.to_dict()
+
+    assert 'spec' in dct
+    assert dct['columns'] == 2
+    expected = {'field': 'color', 'type': 'nominal'}
+    if facet_arg is None or facet_arg == 'facet':
+        assert dct['facet'] == expected
+    else:
+        assert dct['facet'][facet_arg] == expected
